import { addImportsSources, addComponent, useNuxt, defineNuxtModule, createResolver } from '@nuxt/kit';

const libraryName = "vue-map-ui";
const runtimeDir = "./runtime";
const allComponents = [
  "VMapAttributionControl",
  "VMapControlPosition",
  "VMapLayersControl",
  "VMapScaleControl",
  "VMapZoomControl",
  "VMap",
  "VMapPane",
  "VMapArcGisAeroTileLayer",
  "VMapArcGisTileLayer",
  "VMapGoogleTileLayer",
  "VMapMapboxTileLayer",
  "VMapOpenTransportTileLayer",
  "VMapOsmTileLayer",
  "VMapDefaultIcon",
  "VMapDivIcon",
  "VMapDivMarker",
  "VMapIcon",
  "VMapIconMarker",
  "VMapMarker",
  "VMapPinIcon",
  "VMapPinMarker",
  "VMapTileLayer"
];
const allImports = [
  "useMap",
  "usePane",
  "useTileLayer",
  "useMarker",
  "useIcon",
  "useDivIcon",
  "useAttributionControl",
  "useLayersControl",
  "useScaleControl",
  "useZoomControl",
  "useControlPosition"
];
const defaults = {
  components: allComponents,
  imports: allImports,
  autoImports: true,
  leafletStyle: true,
  libraryStyle: true,
  themeStyle: true,
  normalizeStyle: true
};

function resolveImports(config) {
  const { imports } = config;
  const allImports = new Set(imports);
  addImportsSources({
    from: libraryName,
    imports: [...allImports]
  });
}

function isArray(value) {
  return Array.isArray(value);
}
function toArray(value) {
  return isArray(value) ? value : [value];
}

function resolveComponents(config, resolver) {
  const { components } = config;
  const allComponents = new Set(components);
  for (const item of allComponents) {
    const [name, alias] = toArray(item);
    addComponent({
      export: name,
      name: alias || name,
      filePath: libraryName,
      mode: "client"
    });
    addComponent({
      name: alias || name,
      filePath: resolver.resolve(
        runtimeDir,
        "components",
        "ServerPlaceholder.vue"
      ),
      mode: "server"
    });
  }
}

function resolveStyles(config) {
  const nuxt = useNuxt();
  const enabled = isArray(nuxt.options.css);
  if (config.leafletStyle) {
    enabled && nuxt.options.css.push("leaflet/dist/leaflet.css");
  }
  if (config.normalizeStyle) {
    enabled && nuxt.options.css.push(`${libraryName}/dist/normalize.css`);
  }
  if (config.libraryStyle) {
    enabled && nuxt.options.css.push(`${libraryName}/dist/style.css`);
  }
  if (config.themeStyle) {
    enabled && nuxt.options.css.push(`${libraryName}/dist/theme-all.css`);
  }
}

const name = "nuxt-vue-map-ui";
const version = "1.0.11";

const module = defineNuxtModule({
  meta: {
    name,
    version,
    configKey: "vueMap"
  },
  defaults,
  setup(options, nuxt) {
    const resolver = createResolver(import.meta.url);
    resolveStyles(options);
    nuxt.options.imports.autoImport !== false && options.autoImports && resolveImports(options);
    nuxt.options.components !== false && resolveComponents(options, resolver);
  }
});

export { module as default };
